@@ -38,18 +38,16 @@ class MrExpSep_Prior;
 class MrExpSep : public Corr
 {
  private:
-  unsigned int nin;		/* the true input dimension (dim[X]-1) */
   double *d;		        /* kernel correlation width parameter */
   int *b;		        /* dimension-wize linearization */
   double *d_eff;	        /* dimension-wize linearization */
   double *pb;		        /* prob of dimension-wize linearization */
   unsigned int dreject;         /* d rejection counter */
-  double r;                     /* autoregression coefficient */
   double delta;                 /* fine variance discount factor */
-  double nugfine;               /* observation nugget for fine level proc */
+  double nugaux;               /* observation nugget for fine level proc */
   
  public:
-  MrExpSep(unsigned int col, Base_Prior *base_prior);
+  MrExpSep(unsigned int dim, Base_Prior *base_prior);
   virtual Corr& operator=(const Corr &c);
   virtual ~MrExpSep(void);
   virtual void Update(unsigned int n1, unsigned int n2, double **K, double **X, 
@@ -57,19 +55,21 @@ class MrExpSep : public Corr
   virtual void Update(unsigned int n1, double **X);
   virtual void Update(unsigned int n1, double **K, double **X);
   virtual int Draw(unsigned int n, double **F, double **X, double *Z, double *lambda, 
-		   double **bmu, double **Vb, double tau2, double itemp, bool cart, 
+		   double **bmu, double **Vb, double tau2, double itemp,  
 		   void *state);
   virtual void Combine(Corr *c1, Corr *c2, void *state);
   virtual void Split(Corr *c1, Corr *c2, void *state);
   virtual char* State(void);
    virtual unsigned int sum_b(void);
   virtual void ToggleLinear(void);
-  virtual bool DrawNug(unsigned int n, double **X, double **F, double *Z,
+  virtual bool DrawNugs(unsigned int n, double **X, double **F, double *Z,
 		       double *lambda, double **bmu, double **Vb, double tau2, 
-		       double itemp, bool cart, void *state);
+		       double itemp, void *state);
   virtual double* Trace(unsigned int* len);
   virtual char** TraceNames(unsigned int* len);
   virtual void Init(double *dmrexpsep);
+  virtual double* Jitter(unsigned int n1, double **X);
+  virtual double* CorrDiag(unsigned int n1, double **X);
 
   void get_delta_d(MrExpSep* c1, MrExpSep* c2, void *state);
   void propose_new_d(MrExpSep* c1, MrExpSep* c2, void *state);
@@ -83,22 +83,28 @@ class MrExpSep : public Corr
 	     double **Kchol_new, double *log_det_K_new, 
 	     double *lambda_new, double **VB_new, double *bmu_new, 
 	     double *b0, double **Ti, double **T, double tau2, 
-	     double nug, double nugfine, double qRatio, 
+	     double nug, double nugaux, double qRatio, 
 	     double pRatio_log, double a0, double g0, int lin, 
-	     double itemp, bool cart, void *state);
+	     double itemp, void *state);
   double *D(void);
   double Delta(void);
-  double R(void);
-  double Nugfine(void);
+  void SetDelta(double deltanew);
+  void SetNugaux(double nugauxnew);
+  double  CombineNugaux(MrExpSep *c1, MrExpSep *c2, void *state);
+  double  CombineDelta(MrExpSep *c1, MrExpSep *c2, void *state);
+  void SplitNugaux(MrExpSep *c1, MrExpSep *c2, void *state);
+  void SplitDelta(MrExpSep *c1, MrExpSep *c2, void *state);
   void corr_symm(double **K, unsigned int m, double **X, unsigned int n,
-		 double *d, double nug, double nugfine, double r, 
+		 double *d, double nug, double nugaux, 
 		 double delta, double pwr);
   void corr_unsymm(double **K, unsigned int m, 
 		   double **X1, unsigned int n1, double **X2, unsigned int n2,
-		   double *d, double r, double delta, double pwr);
+		   double *d, double delta, double pwr);
   bool DrawDelta(unsigned int n, double **X, double **F, double *Z, double *lambda, 
-		 double **bmu,  double **Vb, double tau2, double itemp, bool cart, 
+		 double **bmu,  double **Vb, double tau2, double itemp,  
 		 void *state);
+
+    double Nugaux(void);
 };
 
 
@@ -111,25 +117,25 @@ class MrExpSep_Prior : public Corr_Prior
 {
 
  private:
-  unsigned int nin;		/* the true input dimension (dim[X]-1) */
   double *d;
   double **d_alpha;	/* d gamma-mixture prior alphas */
   double **d_beta;	/* d gamma-mixture prior beta */
   bool   fix_d;		/* estimate d-mixture parameters or not */
   double d_alpha_lambda[2];	/* d prior alpha lambda parameter */
   double d_beta_lambda[2];	/* d prior beta lambda parameter */
-  double r;                     /* autoregression coefficient */
   double delta;                 /* fine variance discount factor */
-  double nugfine;
+
   double *delta_alpha;
   double *delta_beta;
-  double *nugf_alpha;
-  double *nugf_beta;
+
+  double nugaux;
+  double *nugaux_alpha;
+  double *nugaux_beta;
   
 
  public:
 
-  MrExpSep_Prior(unsigned int col);
+  MrExpSep_Prior(unsigned int dim);
   MrExpSep_Prior(Corr_Prior *c);
   virtual ~MrExpSep_Prior(void);
   virtual void read_double(double *dprior);
@@ -147,20 +153,24 @@ class MrExpSep_Prior : public Corr_Prior
 
   void draw_d_from_prior(double *d_new, void *state);
   double* D(void);
-  double R(void);
   double Delta(void);
-  double Nugfine(void);
+
+  double DeltaDraw(void *state);
+  double NugauxDraw(void *state);
+
   double** DAlpha(void);
   double** DBeta(void);
   double* Delta_alpha(void);
   double* Delta_beta(void);
-  double* Nugf_alpha(void);
-  double* Nugf_beta(void);
   void default_d_priors(void);
   void default_d_lambdas(void);
   double log_Prior(double *d, int *b, double *pb, bool linear);
   double log_DPrior_pdf(double *d);
   void DPrior_rand(double *d_new, void *state);
+
+  double Nugaux(void);
+  double* Nugaux_alpha(void);
+  double* Nugaux_beta(void);
 };
 
 #endif
